<!-- object start -->

{% if search_query_attribute %}
  {% set base_search_query_attribute = search_query_attribute %}
{% else %}
  {% set base_search_query_attribute = "" %}
{% endif %}
<div>
{% if "propertyOrder" not in schema %}
  {% for property_name, property_schema in schema.properties.items() %}
    {% set name = property_name %}
    {% set schema = property_schema %}
    {% if data is not none and property_name in data %}
    {% set data = data[property_name] %}
    {% else %}
    {% set data = none %}
    {% endif %}
    {% if base_search_query_attribute %}
      {% set search_query_attribute = base_search_query_attribute + "." + name %}
    {% else %}
      {% set search_query_attribute = name %}
    {% endif %}
    {% include "objects/view/any.html" %}
  {% endfor %}
{% else %}
  {% for property_name in schema.propertyOrder %}
    {% if property_name in schema.properties %}
      {% set name = property_name %}
      {% set schema = schema.properties[property_name] %}
      {% if data is not none and property_name in data %}
      {% set data = data[property_name] %}
      {% else %}
      {% set data = none %}
      {% endif %}
      {% if base_search_query_attribute %}
        {% set search_query_attribute = base_search_query_attribute + "." + name %}
      {% else %}
        {% set search_query_attribute = name %}
      {% endif %}
      {% include "objects/view/any.html" %}
    {% endif %}
  {% endfor %}
  {% for property_name, property_schema in schema.properties.items() %}
    {% if property_name not in schema.propertyOrder %}
      {% set name = property_name %}
      {% set schema = property_schema %}
      {% if data is not none and property_name in data %}
      {% set data = data[property_name] %}
      {% else %}
      {% set data = none %}
      {% endif %}
      {% if base_search_query_attribute %}
        {% set search_query_attribute = base_search_query_attribute + "." + name %}
      {% else %}
        {% set search_query_attribute = name %}
      {% endif %}
      {% include "objects/view/any.html" %}
    {% endif %}
  {% endfor %}
{% endif %}
</div>
<!-- object end -->