{% if data is not none and "value" in data and not (data.value is none) %}
  {% if data.value %}
    <i class="fa fa-check"></i>
  {% else %}
    <i class="fa fa-times"></i>
  {% endif %}
{% else %}
  &mdash;
{% endif %}
