{% if schema['type'] == 'text' and 'choices' in schema %}
  {% include "objects/forms/form_table_select.html" %}
{% elif schema['type'] == 'text' %}
  {% include "objects/forms/form_table_text.html" %}
{% elif schema['type'] == 'datetime' %}
  {% include "objects/forms/form_table_datetime.html" %}
{% elif schema['type'] == 'quantity' %}
  {% include "objects/forms/form_table_quantity.html" %}
{% elif schema['type'] == 'sample' %}
  {% include "objects/forms/form_table_sample.html" %}
{% elif schema['type'] == 'measurement' %}
  {% include "objects/forms/form_table_measurement.html" %}
{% elif schema['type'] == 'user' %}
  {% include "objects/forms/form_table_user.html" %}
{% elif schema['type'] == 'object_reference' %}
  {% include "objects/forms/form_table_object_reference.html" %}
{#{% elif schema.type == 'object' %}#}
{#  {% include "form_object.html" %}#}
{#{% elif schema.type == 'array' %}#}
{#  {% include "form_array.html" %}#}
{% else %}
  {{ name }}{{ schema.type }}{{ schema['type'] }}
{% endif %}