<div class="form-group row{% if id_prefix+'_text' in errors %} has-error{% elif id_prefix+'_text' in form_data %} has-success{% endif %}" style="padding-right:0.75em">
  <label class="control-label col-md-3">{{ schema.title }}</label>
  <div class="col-md-9">
    {% if 'multiline' in schema and schema['multiline']  %}
      <textarea style="resize: vertical; min-height:2.5em; height:4em;" type="text" class="form-control" name="{{ id_prefix }}_text" placeholder="{{ schema.placeholder or schema.title }}">{% if id_prefix+'_text' in form_data %}{{ form_data[id_prefix+'_text'] }}{% elif data is not none and "text" in data %}{{ data.text }}{% endif %}</textarea>
    {% else %}
      <input type="text" class="form-control" name="{{ id_prefix }}_text" placeholder="{{ schema.placeholder or schema.title }}" {% if id_prefix+'_text' in form_data %}value="{{ form_data[id_prefix+'_text'] }}"{% elif data is not none and "text" in data %}value="{{ data.text }}"{% endif %}>
    {% endif %}
    {% if id_prefix+'_text' in object_errors %}<span class="help-block"><strong>Error:</strong> {{ object_errors[id_prefix+'_text'] }}</span>{% endif %}
    {% if 'note' in schema %}<span class="help-block"><strong>Note:</strong> {{ schema['note'] }}</span>{% endif %}
  </div>
</div>
