{% extends "mails/notifications/base.html" %}

{% block message %}
<p><a href="{{ url_for('.user_profile', user_id=data['inviter_id'], _external=True) }}" style="color:#337ab7;">{{ get_user(data['inviter_id']).name }}</a> has invited you to be a member of the project "<a href="{{ url_for('frontend.project', project_id=data['project_id'], _external=True) }}" style="color:#337ab7;">{{get_project(data['project_id']).name}}</a>". If you wish to join this project, please click the button below:</p>
<p style="text-align: center"><a href="{{ data['confirmation_url'] | safe }}" style="display: inline-block; padding: 6px 12px; margin-bottom: 0; font-size: 14px; font-weight: 400; line-height: 1.42857143; text-align: center; white-space: nowrap; vertical-align: middle; -ms-touch-action: manipulation; touch-action: manipulation; cursor: pointer; -webkit-user-select: none; -moz-user-select: none; -ms-user-select: none; user-select: none; background-image: none; border: 1px solid #122b40; border-radius: 4px; text-decoration: none; color: #f8f8f8; background-color: #023d6b;">Join Project</a></p>
{% endblock %}

{% block notes %}
<p>This invitation will expire at {{ data['expiration_utc_datetime'] }}.</p>
{{ super() }}
{% endblock %}
