<div class="row" style="padding-right:0.75em">
  <label class="col-md-3" style="text-align: right">{{ schema.title }}
    {% if data.text %}
    <a href="{{ url_for('.objects', q=search_query_attribute + ' == "' +data.text + '"', advanced='on') }}" class="search-helper"><i class="fa fa-search" aria-hidden="true"></i></a>
    {% else %}
    <a href="{{ url_for('.objects', q=search_query_attribute + ' == ""', advanced='on') }}" class="search-helper"><i class="fa fa-search" aria-hidden="true"></i></a>
    {% endif %}
  </label>
  <div class="col-md-9">
    {% if 'multiline' in schema and schema['multiline']  %}
      {% if data.text %}
<pre style="font-family: inherit; word-wrap: break-word; white-space: pre-wrap; word-break: normal;">{{ data.text }}</pre>
        {% else %}
          &mdash;
        {% endif %}
    {% else %}
    {{ data.text }}
    {% endif %}
  </div>
</div>
