{% if instrument is not none and instrument.responsible_users %}
<div class="instrument-scientists">
  <h3>Instrument Scientists:</h3>
  <ul>
  {% for user in instrument.responsible_users %}
    {% if not user.is_hidden %}
    <li><a href="{{ url_for('frontend.user_profile', user_id=user.id) }}">{{ user.name }} (#{{ user.id }})</a></li>
    {% endif %}
  {% endfor %}
  </ul>
</div>
{% endif %}
