{% if schema['type'] == 'text' and 'choices' not in schema  %}
  {% include "objects/view/text.html" %}
{% elif schema['type'] == 'text' and 'choices' in schema %}
  {% include "objects/view/select.html" %}
{% elif schema['type'] == 'datetime' %}
  {% include "objects/view/datetime.html" %}
{% elif schema['type'] == 'bool' %}
  {% include "objects/view/bool.html" %}
{% elif schema['type'] == 'quantity' %}
  {% include "objects/view/quantity.html" %}
{% elif schema.type == 'sample' %}
  {% include "objects/view/sample.html" %}
{% elif schema.type == 'measurement' %}
  {% include "objects/view/measurement.html" %}
{% elif schema.type == 'object_reference' %}
  {% include "objects/view/object_reference.html" %}
{% elif schema.type == 'object' %}
  {% include "objects/view/object.html" %}
{% elif schema.type == 'array' %}
  {% include "objects/view/array.html" %}
{% elif schema.type == 'tags' %}
  {% include "objects/view/tags.html" %}
{% elif schema.type == 'hazards' %}
  {% include "objects/view/hazards.html" %}
{% elif schema.type == 'user' %}
  {% include "objects/view/user.html" %}
{% else %}
  {{ name }}{{ schema.type }}{{ schema['type'] }}
{% endif %}
