{% if schema['type'] == 'text' and 'choices' not in schema  %}
  {% include "objects/forms/form_text.html" %}
{% elif schema['type'] == 'text' and 'choices' in schema %}
  {% include "objects/forms/form_select.html" %}
{% elif schema['type'] == 'sample' %}
  {% include "objects/forms/form_sample.html" %}
{% elif schema['type'] == 'measurement' %}
  {% include "objects/forms/form_measurement.html" %}
{% elif schema['type'] == 'object_reference' %}
  {% include "objects/forms/form_object_reference.html" %}
{% elif schema['type'] == 'datetime' %}
  {% include "objects/forms/form_datetime.html" %}
{% elif schema['type'] == 'bool' %}
  {% include "objects/forms/form_bool.html" %}
{% elif schema['type'] == 'quantity' %}
  {% include "objects/forms/form_quantity.html" %}
{% elif schema.type == 'object' %}
  {% include "objects/forms/form_object.html" %}
{% elif schema.type == 'array' %}
  {% include "objects/forms/form_array.html" %}
{% elif schema.type == 'tags' %}
  {% include "objects/forms/form_tags.html" %}
{% elif schema.type == 'hazards' %}
  {% include "objects/forms/form_hazards.html" %}
{% elif schema.type == 'user' %}
  {% include "objects/forms/form_user.html" %}
{% else %}
  {{ name }}{{ schema.type }}{{ schema['type'] }}
{% endif %}