{% extends "mails/base.html" %}

{% block subject %}{{ service_name }} Notification{% endblock %}

{% block content %}
<p>Hello {{ user.name }},</p>

{% block message %}{% endblock %}
{% endblock %}

{% block notes %}
<p>To disable notifications or to receive them on {{ service_name }}, please adjust your <a href="{{ url_for('frontend.user_preferences', user_id=user.id, _external=True) }}" style="color:#337ab7;">notification settings</a>.</p>
{{ super() }}
{% endblock %}
