{% extends "mails/base.html" %}

{% block subject %}{{ service_name }} Email Confirmation{% endblock %}

{% block content %}
<p>Hello,</p>
<p>this email address was assigned to an {{ service_name }} account. To confirm your email address, please click the button below:</p>
<p style="text-align: center"><a href="{{ confirm_url | safe }}" style="display: inline-block; padding: 6px 12px; margin-bottom: 0; font-size: 14px; font-weight: 400; line-height: 1.42857143; text-align: center; white-space: nowrap; vertical-align: middle; -ms-touch-action: manipulation; touch-action: manipulation; cursor: pointer; -webkit-user-select: none; -moz-user-select: none; -ms-user-select: none; user-select: none; background-image: none; border: 1px solid #122b40; border-radius: 4px; text-decoration: none; color: #f8f8f8; background-color: #023d6b;">Confirm Email Address</a></p>
{% endblock %}
