{% extends "base.html" %}

{% block title %}Account is read only — {{ service_name }}{% endblock %}

{% block readonly_alert %}{% endblock %}

{% block content %}
  <h1>Account is read only</h1>
  <p>Your account has been marked as <strong>read only</strong>. If you think this was done erroneously, please <a href="mailto:{{ contact_email }}?subject={{ service_name }}">contact an administrator</a>.</p>
{% endblock %}
