{% if schema['type'] == 'text' %}
  {% include "objects/view/table_text.html" %}
{% elif schema['type'] == 'datetime' %}
  {% include "objects/view/table_datetime.html" %}
{% elif schema['type'] == 'tags' %}
  {% include "objects/view/table_tags.html" %}
{% elif schema['type'] == 'quantity' %}
  {% include "objects/view/table_quantity.html" %}
{% elif schema['type'] == 'sample' %}
  {% include "objects/view/table_sample.html" %}
{% elif schema['type'] == 'measurement' %}
  {% include "objects/view/table_measurement.html" %}
{% elif schema['type'] == 'user' %}
  {% include "objects/view/table_user.html" %}
{% elif schema['type'] == 'object_reference' %}
  {% include "objects/view/table_object_reference.html" %}
{% elif schema['type'] == 'bool' %}
  {% include "objects/view/table_bool.html" %}
{% else %}
  &hellip;
{% endif %}
