<div class="{% if id_prefix+'_text' in errors %}has-error{% elif id_prefix+'_text' in form_data %} has-success{% endif %}" style="padding-right:0.75em">
{% if 'multiline' in schema and schema['multiline']  %}
  <textarea class="form-control" style="resize: vertical;" rows="1" name="{{ id_prefix }}_text" placeholder="{{ schema.placeholder or schema.title }}">{% if id_prefix+'_text' in form_data %}{{ form_data[id_prefix+'_text'] }}{% elif data is not none and "text" in data %}{{ data.text }}{% endif %}</textarea>
{% else %}
  <input type="text" class="form-control" name="{{ id_prefix }}_text" placeholder="{{ schema.placeholder or schema.title }}" {% if id_prefix+'_text' in form_data %}value="{{ form_data[id_prefix+'_text'] }}"{% elif data is not none and "text" in data %}value="{{ data.text }}"{% endif %}>
{% endif %}
</div>