<div class="row" style="padding-right:0.75em">
  <label class="col-md-3" style="text-align: right">{{ schema.title }}
    {% if data is not none and "value" in data %}
      {% if data.value %}
      <a href="{{ url_for('.objects', q=search_query_attribute + ' == True', advanced='on') }}" class="search-helper"><i class="fa fa-search" aria-hidden="true"></i></a></label>
      {% else %}
      <a href="{{ url_for('.objects', q=search_query_attribute + ' == False', advanced='on') }}" class="search-helper"><i class="fa fa-search" aria-hidden="true"></i></a></label>
      {% endif %}
    {% else %}
    <a href="{{ url_for('.objects', q=search_query_attribute, advanced='on') }}" class="search-helper"><i class="fa fa-search" aria-hidden="true"></i></a></label>
    {% endif %}
  <div class="col-md-9">
    {% if data is not none and "value" in data and not (data.value is none) %}
      {% if data.value %}
        <i class="fa fa-check"></i>
      {% else %}
        <i class="fa fa-times"></i>
      {% endif %}
    {% else %}
      &mdash;
    {% endif %}
  </div>
</div>
