<div class="row" style="padding-right:0.75em">
  <label class="col-md-3" style="text-align: right">{{ schema.title }}
    {% if data is not none %}
    <a href="{{ url_for('.objects', q=search_query_attribute + ' == ' + data.utc_datetime.split()[0], advanced='on') }}" class="search-helper"><i class="fa fa-search" aria-hidden="true"></i></a>
    {% else %}
    <a href="{{ url_for('.objects', q=search_query_attribute + ' == 2018-01-01', advanced='on') }}" class="search-helper"><i class="fa fa-search" aria-hidden="true"></i></a>
    {% endif %}
  </label>
  <div class="col-md-9">
    {{ data.utc_datetime }}
  </div>
</div>
