
{% if schema['units'] == "1" %}
  {% if data is not none %}{{ '%g'| format(data.magnitude_in_base_units) }}{% endif %}
{% else %}
  {% if data is not none %}{{ '%g'| format((data | to_datatype).magnitude) }} {{ (data | to_datatype).units | prettify_units }}{% endif %}
{% endif %}
