{% if 'referencing_objects' in related_objects_tree %}{% if related_objects_tree['referenced_objects'] or related_objects_tree['referencing_objects'] %}<input type="checkbox" class="related-objects-tree-toggle" {% if related_objects_tree['path'][0] == related_objects_tree['object_id'] %}checked="checked"{% endif %} id="related_object_toggle_{{ in_export_data_model }}_{{ related_objects_tree['object_id'] }}"><label for="related_object_toggle_{{ in_export_data_model }}_{{ related_objects_tree['object_id'] }}" class="fa fa-fw"></label>{% else %}<i class="fa fa-fw" aria-hidden="true"></i>{% endif %}{% else %}<i class="fa fa-fw fa-ellipsis-h" aria-hidden="true" style="position: relative; z-index:3; background-color: white;"></i>{% endif %}