{% if 'action_type_id' not in schema or schema['action_type_id'] is none %}
  {% set selected_action_type = none %}
{% else %}
  {% set selected_action_type = schema['action_type_id'] %}
{% endif %}
{% if 'action_id' not in schema or schema['action_id'] is none %}
  {% set selected_action = none %}
{% else %}
  {% set selected_action = schema['action_id'] %}
{% endif %}
{% include 'objects/forms/form_object_reference_base.html' %}
